/* 
 *  GGI/DM initialization
 *
 *  Copyright (c) 1999 Nicolai Haehnle - Locinga@pfaffenwinkel.de
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#define DEBUG		1

#include <stdio.h>
#include <string.h>

#include <ggi/internal/dm_int.h>


/* Extension ID. Defaulting to -1 should make segfault on abuse more likely ... */
ggi_extid ggiDMID=-1;

static int changed(ggi_visual_t vis,int whatchanged)
{
	int temp;

	GGIDPRINT("changed() called for DM extension - vis=%p, %i \n",
		vis,whatchanged);

	switch(whatchanged) {
	 case GGI_CHG_APILIST: {
		char api[256],args[256];
		void *err;
		
		GGIDPRINT("changed() APILIST \n");
		for(temp=0;0==ggiGetAPI(vis,temp,api,args);temp++) {
			if (strncmp(api, "display", 7))
				continue;
			strcat(api,"-dm");
			GGIDPRINT("Trying #%d: %s(%s)... ",temp,api,args);
			err=_ggiAddDL(vis, api, args, NULL, GGI_DLTYPE_EXTENSION);
			if (err) {
				GGIDPRINT("Loaded #%d: %s(%s) : %p\n",
					temp,api,args,err);
			} else {
				GGIDPRINT("Load failed\n");
				strcpy(api, "display-default-dm");
				GGIDPRINT("Loading: %s(%s)... ", api, args);
				err=_ggiAddDL(vis,api,args,NULL,GGI_DLTYPE_EXTENSION);
				if (err) {
					GGIDPRINT(": %p\n", err);
				} else {
					GGIDPRINT("Failed\n");
					return -1;
				}
			}
		}
	 } break;
	}

	return 0;
}

/* static int _ggi_error(void) { return -1; } */

int ggiDMInit(void)
{
	ggiDMID=ggiExtensionRegister("GGIDM",sizeof(dmext),changed);
	GGIDPRINT("Initialized DM extension. ID: %i\n",ggiDMID);

	return ggiDMID >= 0 ? 0 : -1;
}

int ggiDMExit(void)
{
	int rc;
	rc=ggiExtensionUnregister(ggiDMID);
	GGIDPRINT("DeInitailized DM extension. rc=%i\n", rc);

	return rc;
}

static void setup(ggi_visual_t vis, dmext *hook)
{
	dmvisual *dv = (dmvisual *)malloc(sizeof(dmvisual));

	GGIDPRINT("LibGGIDM: setup() called\n");

	memset(dv, 0, sizeof(*dv));
	
	hook->dmvis = dv;
	
	dv->parent = hook;
	dv->prev = dv->next = dv;
	dv->vis = vis;
	dv->flags = GGIDM_VISIBLE;
}

static void cleanup(dmext *hook)
{
	dmvisual *dv;
	
	GGIDPRINT("LibGGIDM: cleanup() called\n");

	FREE_LIST(ggidm_drawmap,&hook->drawmaps);
	FREE_LIST(ggidm_drawmap,&hook->fchain);
	
	while(hook->dmvis) {
		dv = hook->dmvis;
		hook->dmvis = (dv != dv->next) ? dv->next : 0;
		dv->prev->next = dv->next;
		dv->next->prev = dv->prev;

		FREE_LIST(ggidm_drawmap,&dv->drawmaps);
		FREE_LIST(dmrect *,&dv->freerects);

		if (!(dv->flags & GGIDM_VISIBLE)) {
			ggiClose(dv->vis);
		}
		free(dv);
	}
}


int ggiDMAttach(ggi_visual_t vis)
{
	int rc;
	dmext *hook;
	
	rc=ggiExtensionAttach(vis,ggiDMID);
	GGIDPRINT("Attached DM extension to %p. rc=%i\n", vis, rc);
	hook = LIBGGI_DMEXT(vis);

	if (rc==0) {	/* We are actually creating the primary instance. */
		memset(hook,0,sizeof(dmext));
		
		setup(vis, hook);

		/* Now fake an "API change" so the right libs get loaded */
		if (changed(vis,GGI_CHG_APILIST)) {
			ggiExtensionDetach(vis,ggiDMID);
			return -1;
		}
	}
	hook->usecount++;

	return rc;
}

int ggiDMDetach(ggi_visual_t vis)
{
	int rc;
	dmext *hook = LIBGGI_DMEXT(vis);
	
	if (!(--(hook->usecount))) {
		cleanup(hook);
	}

	rc=ggiExtensionDetach(vis,ggiDMID);
	GGIDPRINT("Detached DM extension from %p. rc=%i\n", vis, rc);
	
	return rc;
}

